#[macro_use]
extern crate colour;
#[macro_use]
extern crate itertools;
extern crate rand;

mod optimizer;

pub use optimizer::{job_config, swarm_config, PSO};

#[cfg(test)]
mod tests {

    use super::*;
    use job_config::JobConfig;
    use std::f64::consts::PI;
    use swarm_config::{SwarmConfig, TransientBehavior};

    // #[test]
    // fn yep() {
    //     let pso = PSO::default(8, true);
    //     let mut jc = JobConfig::new(30);
    //     jc.max_iterations(1000);

    //     let min = pso.run_job(jc, |x: &[f64]| -> f64 { x.iter().sum() });

    //     println!("minimum of: {}, located at: {:?}", min.0, min.1);
    // }

    #[test]
    fn exp_sin_cost_fn() {
        let num_variables = 3;

        let mut jc = JobConfig::new(num_variables);
        jc.max_iterations(1000);
        jc.update_console(Some(500));
        jc.variable_bound([-2.0 * PI, 2.0 * PI]);

        let mut tb = TransientBehavior::new();
        tb.stochastic_behavior(3.0, 5);
        tb.momentum_mode(0);
        tb.motion_mode(3);

        let mut sc = SwarmConfig::new();
        sc.synergic_behavior(0.4, 128);
        sc.motion_coefficients(0.5, 0.6, 1.0);
        sc.num_particles(128);
        sc.set_transient_behavior(tb);

        let pso = PSO::from_swarm_config(16, true, &sc);

        let min = pso.run_job(jc, move |pt: &[f64]| -> f64 {
            let mut sum = 0.0;
            let mut sin_sum = 0.0;
            for i in 0..num_variables {
                sum += pt[i].abs();
                sin_sum += pt[i].powi(2).sin();
            }
            sum * (-1.0 * sin_sum).exp()
        });

        println!("minimum of: {}, located at: {:?}", min.0, min.1);
    }
}
